/*
 * Copyright 2017-2018, Intel Corporation
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *
 *     * Neither the name of the copyright holder nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef LIBPMEMOBJ_CPP_EXAMPLES_PMPONG_GAMECONTROLLER_HPP
#define LIBPMEMOBJ_CPP_EXAMPLES_PMPONG_GAMECONTROLLER_HPP

#include "GameConstants.hpp"
#include "GameOverView.hpp"
#include "GameView.hpp"
#include "MenuView.hpp"
#include "PongGameStatus.hpp"
#include <libpmemobj++/make_persistent.hpp>
#include <libpmemobj++/p.hpp>
#include <libpmemobj++/persistent_ptr.hpp>
#include <libpmemobj++/pool.hpp>
#include <libpmemobj++/transaction.hpp>

class GameController {
public:
	GameController();
	~GameController();

	void gameLoop(bool isSimulation = false);

private:
	pmem::obj::persistent_ptr<PongGameStatus> gameStatus;

	void gameOver(sf::RenderWindow *gameWindow, View *view);
	void menu(sf::RenderWindow *gameWindow, View *view);
	void handleEventKeypress(sf::Event &event,
				 sf::RenderWindow *gameWindow);
	void gameMatch(sf::RenderWindow *gameWindow, View *view);
	void gameMatchSimulation(sf::RenderWindow *gameWindow, View *view);

	void resetGameStatus();
};

#endif /* LIBPMEMOBJ_CPP_EXAMPLES_PMPONG_GAMECONTROLLER_HPP */
